import { world, system } from "@minecraft/server";

const mana = world.scoreboard.getObjective("mana") ?? world.scoreboard.addObjective("mana","MANA")
const data = world.scoreboard.getObjective("data") ?? world.scoreboard.addObjective("data","DATA")

let bllocation;
let Position;

function toDegree(radian) {
  return (radian * 180) / Math.PI;
}

function viewDirectionToRotation(viewDirection) {
  const { x: vx, y: vy, z: vz } = viewDirection;
  const x = toDegree(Math.asin(-vy));
  const y = toDegree(Math.atan2(-vx, vz));
  return { x, y };
}

system.runInterval(() => {
    for (const player of world.getAllPlayers()) {
    const scoresMana = mana.getScore(player);
    if (!player.hasTag("dlay") && player.hasTag("water") && scoresMana >= 15 ){
        player.runCommandAsync("execute at @s[tag=water] run execute at @e[tag=!water,r=10] run particle tunekeshi:water_particle ~~~")
        player.runCommandAsync("function water")
        player.addTag("dlay");
        mana.addScore(player,-15)
        system.runTimeout(() => {
            player.removeTag("dlay");
        },5);
    }
    if (!player.hasTag("dlay") && player.hasTag("air") && scoresMana >= 5 ){
        player.runCommandAsync("function air")
        player.addTag("dlay");
        mana.addScore(player,-5)
        system.runTimeout(() => {
            player.removeTag("dlay");
        },5);
    }
    if (!player.hasTag("dlay") && player.hasTag("dark") && scoresMana >= 5 ){
        player.runCommandAsync("function dark")
        player.addTag("dlay");
        mana.addScore(player,-5)
        system.runTimeout(() => {
            player.removeTag("dlay");
        },5);
    }
    data.addScore(player,1);
    player.runCommandAsync(`scoreboard players add @a[scores={data=10..}] mana 1`);
    player.runCommandAsync(`scoreboard players set @a[scores={mana=101..}] mana 100`);
    player.runCommandAsync(`scoreboard players set @a[scores={mana=..0}] mana 0`);
    player.runCommandAsync(`scoreboard players set @a[scores={data=10..}] data 0`);
    player.runCommandAsync(`title @s[hasitem={location=slot.weapon.mainhand,item=tunekeshi:magic_wand}] actionbar §bMANA §f: ${scoresMana}`)
    player.runCommandAsync(`scoreboard players add @s[hasitem={location=slot.armor.head,item=tunekeshi:bousi}] data 1`)
    player.runCommandAsync(`scoreboard players add @s[hasitem={location=slot.armor.chest,item=tunekeshi:fuku}] data 1`)
}});

world.afterEvents.entityDie.subscribe((ev) => {
    const { deadEntity:entity }  = ev;
    if ( entity.typeId == "minecraft:player" && entity.hasTag("water")) {
        player.removeTag("water");
    }
    if ( entity.typeId == "minecraft:player" && entity.hasTag("dark")) {
        player.removeTag("dark");
    }
    if ( entity.typeId == "minecraft:player" && entity.hasTag("air")) {
        player.removeTag("air");
    }
});

world.afterEvents.itemStopUse.subscribe(ev => { 
    const { itemStack, source } = ev;
    const Bookoffhand = source.getComponent("minecraft:equippable").getEquipment("Offhand")
    if (itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book2") {
        source.removeTag("water");
    }
    if (itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book3") {
        source.removeTag("air");
    }
    if (itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book4") {
        source.removeTag("dark");
    }
});

world.afterEvents.itemUse.subscribe(event => {
    const { itemStack, source } = event;
    const scoresMana = mana.getScore(source);
    const Bookoffhand = source.getComponent("minecraft:equippable").getEquipment("Offhand"); 
    const head = source.getHeadLocation(); 
    const view = source.getViewDirection();
    if (!source.hasTag("cooldown") && itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book17" && scoresMana >= 15) {
        for (let i = 1; i < 56; i++) {
            const blocklocation = {
                x: head.x + view.x * i,
                y: head.y + view.y * i,
                z: head.z + view.z * i
            };
            const blocklocationplus = {
                x: head.x + view.x * i,
                y: (head.y + view.y * i) + 1,
                z: head.z + view.z * i
            };
            const block = source.dimension.getBlock(blocklocation);
            const blockplus = source.dimension.getBlock(blocklocationplus);
            if (!block.isAir && blockplus.isAir) {
                bllocation = {
                    x: block.location.x,
                    y: block.location.y + 1,
                    z: block.location.z
                }
                source.addTag("cooldown")
                source.teleport(bllocation)
                mana.addScore(source, -15)
                system.runTimeout(() => {
                  source.removeTag("cooldown")
                },30);
            }
        }
    }
    if (itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book1" && scoresMana >= 10 ){
        source.addTag("fire")
        mana.addScore(source, -10)
        source.runCommandAsync("function fire")
    }
    if (itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book6" && scoresMana >= 30 ){
        mana.addScore(source,-30)
        source.runCommandAsync("execute at @e[r=10,type=!player,type=!tunekeshi:magma_mahou] run summon tunekeshi:magma_mahou")
    }
    if (itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book7" && scoresMana >= 15 ){
        const spawnLocation = {
            x: head.x + view.x,
            y: head.y + view.y,
            z: head.z + view.z
        };
        mana.addScore(source,-15)
        const projectile = source.dimension.spawnEntity("tunekeshi:fire3",spawnLocation);
        const viewDirection = source.getViewDirection();
        const { x: rx, y: ry } = viewDirectionToRotation(viewDirection);
        projectile.applyImpulse({x:viewDirection.x*2,y:viewDirection.y*2,z:viewDirection.z*2});
        projectile.setRotation({ x: -rx, y: -ry });
    }
    if (itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book2" && scoresMana >= 15 ){
        source.addTag("water")
    }
    if (!source.hasTag("cooldown") && itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book8" && scoresMana >= 25 ){
        mana.addScore(source,-25)
        source.addTag("cooldown")
        source.runCommandAsync("execute at @e[type=!player,type=!tunekeshi:ice,r=10] run summon tunekeshi:ice")
        system.runTimeout(() => {
            source.removeTag("cooldown")
        },30);
    }
    if (!source.hasTag("cooldown") && itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book9" && scoresMana >= 30 ){
        const spawnLocation = {
            x: head.x + view.x,
            y: head.y + view.y,
            z: head.z + view.z
        };
        mana.addScore(source,-30)
        const projectile = source.dimension.spawnEntity("tunekeshi:ice3",spawnLocation);
        const viewDirection = source.getViewDirection();
        const { x: rx, y: ry } = viewDirectionToRotation(viewDirection);
        projectile.applyImpulse({x:viewDirection.x*2,y:viewDirection.y*2,z:viewDirection.z*2});
        projectile.setRotation({ x: -rx, y: -ry });
        system.runTimeout(() => {
            source.removeTag("cooldown")
        },30);
    }
    if (itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book3" && scoresMana >= 5){
        source.addTag("air")
    }
    if (!source.hasTag("cooldown") && itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book10" && scoresMana >= 15){
        mana.addScore(source,-15)
        source.addTag("air2")
        source.addTag("cooldown")
        source.runCommandAsync("function air2")
        system.runTimeout(() => {
            source.removeTag("cooldown")
        },30)
    }
    if (itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book4" && scoresMana >= 5){
        source.addTag("dark")
    }
    if (!source.hasTag("cooldown") && itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book11" && scoresMana >= 25){
        const spawnLocation = {
            x: head.x + view.x,
            y: head.y + view.y,
            z: head.z + view.z
        };
        mana.addScore(source,-25)
        source.addTag("cooldown")
        const projectile = source.dimension.spawnEntity("tunekeshi:dark3",spawnLocation);
        const viewDirection = source.getViewDirection();
        const { x: rx, y: ry } = viewDirectionToRotation(viewDirection);
        projectile.applyImpulse({x:viewDirection.x*2,y:viewDirection.y*2,z:viewDirection.z*2});
        projectile.setRotation({ x: -rx, y: -ry });
        system.runTimeout(() => {
            source.removeTag("cooldown")
        },30)
    }
    if (itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book5" && scoresMana >= 15){
        mana.addScore(source,-15)
        source.addTag("light")
        source.runCommandAsync("function light")
    }
    if (itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book12" && scoresMana >= 20){
        mana.addScore(source,-20)
        source.runCommandAsync("function light2")
    }
    if (!source.hasTag("cooldown") && itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book13" && scoresMana >= 5){
        mana.addScore(source,-5)
        source.addTag("cooldown")
        source.runCommandAsync("scoreboard players add @a[r=10] mana 20")
        system.runTimeout(() => {
            source.removeTag("cooldown")
        },30)
    }
    if (itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book14" && scoresMana >= 20){
        mana.addScore(source,-20)
        source.runCommandAsync("effect @e[r=10,type=!player] fatal_poison 3 255 true")
    }
    if (itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book15" && scoresMana >= 20){
        mana.addScore(source,-20)
        source.runCommandAsync("particle tunekeshi:knockback_particle ~~~")
        source.runCommandAsync("summon tunekeshi:knockback")
    }
    if (itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book16" && scoresMana >= 15){
        mana.addScore(source,-15)
        source.runCommandAsync("function summonmikata")
    }
    if (!source.hasTag("cooldown") && itemStack.typeId == "tunekeshi:magic_wand" && Bookoffhand.typeId == "tunekeshi:magic_book18" && scoresMana >= 25){
        mana.addScore(source,-25)
        source.addTag("cooldown")
        source.runCommandAsync("execute at @s run particle tunekeshi:white_bomb_particle ~~~")
        source.runCommandAsync("execute at @e[r=10,type=!player] run particle tunekeshi:white_particle ~~~")
        source.runCommandAsync("execute at @e[r=10,type=!player] run summon lightning_bolt")
        system.runTimeout(() => {
        source.runCommandAsync("execute at @s run particle tunekeshi:white_bomb_particle ~~~")
        source.runCommandAsync("execute at @e[r=10,type=!player] run particle tunekeshi:white_particle ~~~")
        source.runCommandAsync("execute at @e[r=10,type=!player] run summon lightning_bolt")
        },30)
        system.runTimeout(() => {
        source.runCommandAsync("execute at @s run particle tunekeshi:white_bomb_particle ~~~")
        source.runCommandAsync("execute at @e[r=10,type=!player] run particle tunekeshi:white_particle ~~~")
        source.runCommandAsync("execute at @e[r=10,type=!player] run summon lightning_bolt")
        source.runCommandAsync("execute at @e[r=10,type=!player] run summon tunekeshi:white_bomb")
        },50)
        system.runTimeout(() => {
            source.removeTag("cooldown")
        },100)
    }});